function [x] = cgtv(y, alpha, beta, Nit)

% [x, cost] = CGTV(y, alpha, beta, Nit)
%
% Compound and generalized TV (using 1st and 3rd order derivatives)
% solved by using MM algorithm
%
% Input:
%   y    : input data
%   alpha: regularizer parameter (for 1st order derivative component)
%   beta : regularizer parameter (for 3rd order derivative component)
%   Nit  : number of iterations
% 
% Output:
%   x    : output data
%
% Cost function:
%   J(x) = 0.5||y - x||_2^2 + alpha||D1 x||_1 + beta||D3 x||_1

% Reference
% Detection of normal and slow saccades using implicit piecewise polynomial approximation
% Weiwei Dai, Ivan Selesnick, John-Ross Rizzo, Janet Rucker, Todd Hudson
%
% Tandon School of Engineering and School of Medicine
% New York University
% New York, USA
%
% Contact: 
% Ivan Selesnick selesi@nyu.edu
% Weiwei Dai wd471@nyu.edu


EPS = 1E-10;                % smoothed penalty function
psi = @(x) sqrt(x.^2 + EPS);

y = y(:);                   % ensure column vector

N = length(y);

e = ones(N, 1);
D1 = spdiags([-e e], [0 1], N-1, N);
D3 = spdiags([-e 3*e -3*e e], [0 1 2 3], N-3, N);
I = spdiags(e, 0, N, N);

x = y;                      % initialization  

for k = 1:Nit
    Lam1 = spdiags(alpha./psi(diff(x)), 0, N-1, N-1);
    Lam3 = spdiags(beta./psi(diff(x,3)), 0, N-3, N-3);
    temp = I + D1'*Lam1*D1 + D3'*Lam3*D3;
    x = temp\y;
end

end

